#ifndef _ASSIGN_VECTor-TO_ZERO_HPP_
//! \cond
#define _ASSIGN_VECTOR_TO_ZERO_HPP_ "$Id: assignVectorToZero.hpp 434 2020-07-21 18:19:20Z geoff $"
//! \endcond

#include <utils/base/assignToZero.hpp>
#ifdef _WIN32
#include <intrin.h>
#else
#include <xmmintrin.h>
#endif

/** \addtogroup FARGOSbase
 * @{
 */

// subgroup
/** \addtogroup assignToZero
 * @{
 */
/* implementations of assignToZero for common
 * fundamental signed numeric types
 */

#ifndef _WIN32
// 4-elements of 32-bit integers
//! \brief Assigns 0 to a 4-element vector of 32-bit integers
extern void assignToZero(__v4si &n) OME_ALWAYS_INLINE;
inline void assignToZero(__v4si &n) {
    // zero out by doing xor against self
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wmaybe-uninitialized" 
    n = (__v4si) _mm_xor_si128((__m128i) n, (__m128i) n); // NOTE: ignore warnings about being n uninitialized -- anything XORd with itself is 0
#pragma GCC diagnostic pop
}
#endif

// 4-elements of 32-bit integers
//! \brief Assigns 0 to a 128-bit integr vector element
extern void assignToZero(__m128i &n) OME_ALWAYS_INLINE;
inline void assignToZero(__m128i &n) {
    // zero out by doing xor against self
    n = _mm_xor_si128(n, n);
}

/** @} */

/** @} */
#endif
/* vim: set expandtab shiftwidth=4 tabstop=4: */
